@@ -97,17 +97,23 @@ bool            enable_trace       = false;
 
 bool            enable_sm          = true;
 bool            enable_bench       = true;
+bool            enable_mch_read    = true;
+
+bool            enable_ecc_polling = false;
 
 bool            pause_at_start     = true;
 
 power_save_t    power_save         = POWER_SAVE_HIGH;
 
 bool            enable_tty         = false;
-int             tty_params_port    = SERIAL_PORT_0x3F8;
-int             tty_params_baud    = SERIAL_DEFAULT_BAUDRATE;
-int             tty_update_period  = 2;     // Update TTY every 2 seconds (default)
+uintptr_t       tty_address        = 0x3F8;             // Legacy IO or MMIO Address accepted
+int             tty_baud_rate      = 115200;
+int             tty_update_period  = 2;                 // Update TTY every 2 seconds (default)
 
-bool            err_banner_redraw  = false; // Redraw banner on new errors (if previsouly removed)
+uint32_t        tty_mmio_ref_clk   = UART_REF_CLK_MMIO; // Reference clock for MMIO (in Hz)
+int             tty_mmio_stride    = 4;                 // Stride for MMIO (register width in bytes)
+
+bool            err_banner_redraw  = false;             // Redraw banner on new errors
 
 //------------------------------------------------------------------------------
 // Private Functions
@@ -122,6 +128,28 @@ static void parse_serial_params(const char *params)
         return;
     }
 
+    // Check if console is MMIO and grab address and stride
+    uintptr_t mmio_adr = 0;
+    if (strncmp(params, "mmio,0x", 7) == 0) {
+        mmio_adr = hexstr2int(params+7);
+        tty_mmio_stride = 1;
+    } else if (strncmp(params, "mmio16,0x", 9) == 0) {
+        mmio_adr = hexstr2int(params+9);
+        tty_mmio_stride = 2;
+    } else if (strncmp(params, "mmio32,0x", 9) == 0) {
+        mmio_adr = hexstr2int(params+9);
+        tty_mmio_stride = 4;
+    }
+
+    if(strncmp(params, "mmio", 4) == 0) {
+        if (mmio_adr > 0xFFFF) {
+            tty_address = mmio_adr;
+        } else {
+            enable_tty = false;
+        }
+        return;
+    }
+
     // No TTY port passed, use default ttyS0
     if (strncmp(params, "ttyS", 5) == 0) {
         return;
@@ -129,7 +157,7 @@ static void parse_serial_params(const char *params)
 
     // Configure TTY port or use default
     if (params[4] >= '0' && params[4] <= '3') {
-        tty_params_port = params[4] - '0';
+        tty_address = serial_io_ports[params[4] - '0'];
     } else {
         return;
     }
@@ -144,27 +172,27 @@ static void parse_serial_params(const char *params)
         default:
             return;
         case '1':
-            tty_params_baud   = (params[7] == '9') ? 19200 : 115200;
+            tty_baud_rate   = (params[7] == '9') ? 19200 : 115200;
             tty_update_period = (params[7] == '9') ? 4 : 2;
             break;
         case '2':
-            tty_params_baud   = 230400;
+            tty_baud_rate   = 230400;
             tty_update_period = 2;
             break;
         case '3':
-            tty_params_baud   = 38400;
+            tty_baud_rate   = 38400;
             tty_update_period = 4;
             break;
         case '5':
-            tty_params_baud   = 57600;
+            tty_baud_rate   = 57600;
             tty_update_period = 3;
             break;
         case '7':
-            tty_params_baud   = 76800;
+            tty_baud_rate   = 76800;
             tty_update_period = 3;
             break;
         case '9':
-            tty_params_baud   = 9600;
+            tty_baud_rate   = 9600;
             tty_update_period = 5;
             break;
     }
@@ -209,6 +237,8 @@ static void parse_option(const char *option, const char *params)
         enable_big_status = false;
     } else if (strncmp(option, "noehci", 7) == 0) {
         usb_init_options |= USB_IGNORE_EHCI;
+    } else if (strncmp(option, "nomch", 6) == 0) {
+        enable_mch_read = false;
     } else if (strncmp(option, "nopause", 8) == 0) {
         pause_at_start = false;
     } else if (strncmp(option, "nosm", 5) == 0) {
@@ -812,8 +842,6 @@ void config_init(void)
         cpu_state[i] = CPU_STATE_ENABLED;
     }
 
-    enable_temperature &= !no_temperature;
-
     power_save = POWER_SAVE_HIGH;
 
     const boot_params_t *boot_params = (boot_params_t *)boot_params_addr;
@@ -849,9 +877,9 @@ void config_menu(bool initial)
             if (!smp_enabled)  set_foreground_colour(BOLD+BLACK);
             prints(POP_R+7,  POP_LI, "<F5>  CPU selection");
             if (!smp_enabled)  set_foreground_colour(WHITE);
-            if (no_temperature) set_foreground_colour(BOLD+BLACK);
+            //if (no_temperature) set_foreground_colour(BOLD+BLACK);
             printf(POP_R+8,  POP_LI, "<F6>  Temperature %s", enable_temperature ? "disable" : "enable ");
-            if (no_temperature) set_foreground_colour(WHITE);
+            //if (no_temperature) set_foreground_colour(WHITE);
             printf(POP_R+9,  POP_LI, "<F7>  Boot trace %s",  enable_trace  ? "disable" : "enable ");
             prints(POP_R+10, POP_LI, "<F10> Exit menu");
         } else {
@@ -890,9 +918,7 @@ void config_menu(bool initial)
             break;
           case '6':
             if (initial) {
-                if (!no_temperature) {
-                    enable_temperature = !enable_temperature;
-                }
+                enable_temperature = !enable_temperature;
             }
             break;
           case '7':
