@@ -58,14 +58,19 @@ extern bool         enable_trace;
 extern bool         enable_sm;
 extern bool         enable_tty;
 extern bool         enable_bench;
+extern bool         enable_mch_read;
+extern bool         enable_ecc_polling;
 
 extern bool         pause_at_start;
 
 extern power_save_t power_save;
 
-extern int tty_params_port;
-extern int tty_params_baud;
-extern int tty_update_period;
+extern uintptr_t    tty_address;
+extern int          tty_baud_rate;
+extern int          tty_update_period;
+
+extern uint32_t     tty_mmio_ref_clk;
+extern int          tty_mmio_stride;
 
 extern bool err_banner_redraw;
 
