@@ -28,6 +28,7 @@
 #include "io.h"
 #include "keyboard.h"
 #include "pmem.h"
+#include "memctrl.h"
 #include "memsize.h"
 #include "pci.h"
 #include "screen.h"
@@ -235,6 +236,8 @@ static void global_init(void)
 
     config_init();
 
+    memctrl_init();
+
     tty_init();
 
     smp_init(smp_enabled);
