@@ -50,7 +50,11 @@
 #define IMC_ADL_N       0x3061  // Core 12th Gen (Alder Lake-N - Gracemont E-Cores only)
 
 #define IMC_BYT         0x4010  // Atom Bay Trail
-#define IMC_CDT         0x4020  // Atom Cedar Trail
+#define IMC_SLT         0x4020  // Atom Silverthorne / Diamondville
+#define IMC_PNV         0x4030  // Atom Pineview
+#define IMC_CLT         0x4040  // Atom Clover Trail / Cloverview
+#define IMC_CDT         0x4050  // Atom Cedar Trail / Cedarview
+#define IMC_TNC         0x4060  // Atom Tunnel Creek / Lincroft
 
 #define IMC_K8          0x8000  // Old K8
 #define IMC_K10         0x8010  // K10 (Family 10h & 11h)
@@ -60,20 +64,21 @@
 #define IMC_K16         0x8050  // Kabini & related (Family 16h)
 #define IMC_K17         0x8060  // Zen & Zen2 (Family 17h)
 #define IMC_K18         0x8070  // Hygon (Family 18h)
-#define IMC_K19         0x8080  // Zen3 (Family 19h)
+#define IMC_K19_VRM     0x8080  // Zen3 (Family 19h - Vermeer)
 #define IMC_K19_CZN     0x8081  // Cezanne APU
-#define IMC_K19_RPL     0x8091  // Zen4 (Family 19h)
+#define IMC_K19_CHL     0x8090  // Zen3 Chagall TR
+
+#define IMC_K19_RBT     0x8100  // Zen3+ (Rembrandt)
+#define IMC_K19_RPL     0x8110  // Zen4 (Raphael)
+#define IMC_K19_PHX     0x8120  // Zen4 (Phoenix)
+#define IMC_K19_STK     0x81A0  // Zen4 (Storm Peak)
+#define IMC_K19_GRG     0x8150  // Zen5 (Granite Ridge)
 
 /**
  * A string identifying the CPU make and model.
  */
 extern const char *cpu_model;
 
-/**
- * A number identifying the integrated memory controller type.
- */
-extern uint16_t imc_type;
-
 /**
  * The size of the L1 cache in KB.
  */
@@ -109,11 +114,6 @@ extern uint32_t l3_cache_speed;
  */
 extern uint32_t ram_speed;
 
-/**
- * A flag indicating that we can't read the core temperature on this CPU.
- */
-extern bool no_temperature;
-
 /**
  * The TSC clock speed in kHz. Assumed to be the nominal CPU clock speed.
  */
