@@ -10,12 +10,12 @@
 #define JEP106_CNT \
     sizeof(jep106)/sizeof(jep106[0])
 
-struct spd_jedec_manufacturer {
+struct __attribute__((packed)) spd_jedec_manufacturer {
     uint16_t jedec_code;
     char *name;
 };
 
-static const struct spd_jedec_manufacturer jep106[] = {
+static const struct __attribute__((packed)) spd_jedec_manufacturer jep106[] = {
     { 0x0001, "AMD" },
 //  { 0x0002, "AMI" },
 //  { 0x0003, "Fairchild" },
@@ -478,7 +478,7 @@ static const struct spd_jedec_manufacturer jep106[] = {
 //  { 0x0353, "Primarion" },
 //  { 0x0354, "Picochip Designs Ltd" },
 //  { 0x0355, "Silverback Systems" },
-//  { 0x0356, "Jade Star Technologies" },
+    { 0x0356, "Jade Star" },
 //  { 0x0357, "Pijnenburg Securealink" },
     { 0x0358, "takeMS" }, // Ultron AG
 //  { 0x0359, "Cambridge Silicon Radio" },
@@ -882,7 +882,7 @@ static const struct spd_jedec_manufacturer jep106[] = {
 //  { 0x066E, "Certicom Corporation" },
 //  { 0x066F, "JSC ICC Milandr" },
 //  { 0x0670, "PhotoFast Global Inc" },
-//  { 0x0671, "InnoDisk Corporation" },
+    { 0x0671, "InnoDisk" },
 //  { 0x0672, "Muscle Power" },
 //  { 0x0673, "Energy Micro" },
 //  { 0x0674, "Innofidei" },
